param (
    [String] $Secret = $null,
    [String] $WebState = $null,
    [Switch] $help = $false,
    [Switch] $ActiveMQ = $false,
    [Switch] $All = $false,
    [Switch] $Base = $false,
    [Switch] $Everything = $false,
    [Switch] $Gov = $false,
    [Switch] $Governance = $false,
    [Switch] $IDGov = $false,
    [Switch] $Java = $false,
    [Switch] $JDK = $false,
    [Switch] $JRE = $false,
    [Switch] $Mail = $false,
    [Switch] $PG = $false,
    [Switch] $Postgres = $false,
    [Switch] $PostgreSQL = $false,
    [Switch] $Tomcat = $false,
    [Switch] $Web = $false
)

Add-Type -AssemblyName System.IO.Compression.FileSystem

#Versions
$VSN_ACTIVEMQ           = "5.17.6"
$VSN_JDK                = "11.68.17-ca-jdk11.0.21"
$VSN_POSTGRESQL         = "14.9-1"
$VSN_POSTGRESQL_SERVICE = "14"
$VSN_TOMCAT             = "9.0.83"

#Names
$NAME_ACTIVEMQ          = "activemq"
$NAME_POSTGRES          = "postgres"
$NAME_TOMCAT            = "tomcat"
$NAME_TOMCAT_LAUNCH     = "start-tomcat.bat"
$NAME_TOMCAT_QUIT       = "stop-tomcat.bat"
$NAME_TOMCAT_SERVICE    = "TomcatService.exe"

#Directories
$DIR_ACTIVEMQ           = "${NAME_ACTIVEMQ}"
$DIR_BACKUP             = "backup"
$DIR_BASE               = "c:\netiq\idm\apps"
$DIR_COMPRESSED         = "${PSScriptRoot}\compressed"
$DIR_JDK                = "jdk"
$DIR_LOG_BASE           = "${PSScriptRoot}"
$DIR_LOG_PARENT         = "logs"
$DIR_LOG                = "${DIR_LOG_BASE}\${DIR_LOG_PARENT}"
$DIR_POSTGRES           = "${NAME_POSTGRES}"
$DIR_POSTGRES_DATA      = "data"
$DIR_PRISTINE           = "${PSScriptRoot}\untouched"
$DIR_TOMCAT             = "${NAME_TOMCAT}"

#Files
$FILE_ACTIVEMQ          = "${DIR_COMPRESSED}\apache-activemq-${VSN_ACTIVEMQ}-bin.zip"
$FILE_JDK               = "${DIR_COMPRESSED}\zulu${VSN_JDK}-win_x64.zip"
$FILE_POSTGRESQL        = "${DIR_COMPRESSED}\postgresql-${VSN_POSTGRESQL}-windows-x64.exe"
$FILE_LOG               = "${DIR_LOG}\IDGov-Core-Helper.log"
$FILE_TOMCAT            = "${DIR_COMPRESSED}\apache-tomcat-${VSN_TOMCAT}-windows-x64.zip"

#Services (final state can be `start`, `stop`, and `restart`)
$SERV_TOMCAT_SERVICE    = "IDM Apps Tomcat Service"
$SERV_TOMCAT_STATE      = ""

#Parameters
$PARAM_OSP_LDAP_TYPE    = "edir"
$PARAM_OSP_LOGGING      = "WARN"
$PARAM_POSTGRES_PORT    = "5432"
$PARAM_TOMCAT_SERVER    = "localhost"
$PARAM_TOMCAT_OPTS_COMMON_ALL = "-Dnetiqtomcatcustomsrvidentify=netiqtomcatcustomsrvidentify;-Djava.awt.headless=true;-Dcom.netiq.ism.config=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;"
$PARAM_TOMCAT_OPTS_OSP  = "-Dcom.netiq.osp.ext-context-file=${DIR_BASE}\osp\lib\osp-conf-${PARAM_OSP_LDAP_TYPE}.jar;-Dcom.netiq.idm.osp.logging.level=${PARAM_OSP_LOGGING};-Dcom.netiq.idm.osp.client.host=${PARAM_TOMCAT_SERVER};-Dcom.netiq.idm.osp.logging.file.dir=${DIR_BASE}\${DIR_TOMCAT}\logs;-Dcom.netiq.idm.osp.audit.enabled=false;"
$PARAM_TOMCAT_OPTS_COMMON_IDENT = "-Dlog4j.configuration=file:${DIR_BASE}\${DIR_TOMCAT}\conf\log4j.properties;-Dinternal.atlaslite.jcce.xml.w3c.XMLUtil.suppressSecurityWarning=true;"
$PARAM_TOMCAT_OPTS_GOV  = "-Dcom.netiq.ism.config.is.jndi.env=true;-Dnovell.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_OPTS_RPT  = "-Dcom.netiq.rpt.config.file=${DIR_BASE}\${DIR_TOMCAT}\conf\ism-configuration.properties;-Dextend.local.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;-Didmuserapp.logging.config.dir=${DIR_BASE}\${DIR_TOMCAT}\conf;"
$PARAM_TOMCAT_JVM_OPTS  = "${PARAM_TOMCAT_OPTS_COMMON_ALL}${PARAM_TOMCAT_OPTS_OSP}${PARAM_TOMCAT_OPTS_COMMON_IDENT}${PARAM_TOMCAT_OPTS_GOV}${PARAM_TOMCAT_OPTS_RPT}"
$PARAM_TOMCAT_JVM_MEM_INIT  = "1024"
$PARAM_TOMCAT_JVM_MEM_FINAL = "1024"

#Parse and consume input arguments provided
$USAGE_THIS = "Usage: " + $MyInvocation.MyCommand.Name + " -all|-gov|-base|-tomcat|-activeMQ|-postgreSQL|-java [<PG password>]"

$PARAM_INSTALL_ACTIVE_MQ = $false
$PARAM_INSTALL_JAVA = $false
$PARAM_INSTALL_POSTGRESQL = $false
$PARAM_INSTALL_WEB = $false

If ($help) {
    Write-Host "$USAGE_THIS"
    Exit
}
If ($All -or $Everything) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_POSTGRESQL = $true
    $PARAM_INSTALL_WEB = $true
}
If ($Governance -or $Gov -or $IDGov) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_WEB = $true
}
If ($Base -or $Tomcat -or $Web) {
    $PARAM_INSTALL_JAVA = $true
    $PARAM_INSTALL_WEB = $true
}
If ($ActiveMQ -or $Mail) {
    $PARAM_INSTALL_ACTIVE_MQ = $true
    $PARAM_INSTALL_JAVA = $true
}
If ($Java -or $JDK -or $JRE) {
    $PARAM_INSTALL_JAVA = $true
}
If ($Postgres -or $PostgreSQL -or $PG) {
    $PARAM_INSTALL_POSTGRESQL = $true
}

If (-not $WebState.equals("")) {
    $SERV_TOMCAT_STATE = $WebState
}

If (-not $Secret.equals("")) {
    $PARAM_POSTGRES_SECRET = $Secret
}

If (-not $PARAM_INSTALL_ACTIVE_MQ -and -not $PARAM_INSTALL_JAVA -and -not $PARAM_INSTALL_POSTGRESQL -and -not $PARAM_INSTALL_WEB) {
    Write-Host "Nothing specified for installation!"
    Write-Host "$USAGE_THIS"
    Exit
}
If ($PARAM_INSTALL_POSTGRESQL -and $PARAM_POSTGRES_SECRET -eq $null) {
    Write-Host "Required PostgreSQL password missing!"
    Write-Host "$USAGE_THIS"
    Exit
}

Function Unzip([string]${zip}, [string]${destination})
{
    ${private:parent} = Split-Path -Path ${destination} -Parent
    ${private:leaf} = Split-Path -Path ${destination} -Leaf
    If (-not (Test-Path ${destination})) {
        ${null} = (New-Item -Path "${parent}" -Name "${leaf}" -ItemType "directory")
    }
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Extracting `"${zip}`" into `"${destination}`":"
    Write-Host "######################################################################################"
    [System.IO.Compression.ZipFile]::ExtractToDirectory(${zip}, ${destination})
    ${private:children} = @(Get-ChildItem -Path "${destination}")
    If (($(${children}.count) -eq 1) -and (${children}[0].Attributes -match "Directory")) {
        # When the only child-item is a directory we should strip off a layer, which should be safe
        #   Otherwise the (extracted) files in this folder are those that should stay
        # On some systems we cannot rename the extracted folder immediately
        ${private:rename_waiting} = ${TRUE}
        ${private:rename_count} = 0
        While (${private:rename_waiting}) {
            try {
                Rename-Item -Path "$(${children}[0].FullName)" -newName "${leaf}" -ErrorAction Stop
                ${private:rename_waiting} = ${FALSE}
            } catch [System.IO.IOException] {
                If (${private:rename_count}++ -ge 15) {
                    # We have waited long enough; report the error
                    ${private:rename_waiting} = ${FALSE}
                    throw $_
                } Else {
                    Start-Sleep -Seconds 2
                }
            }
        }
        Get-ChildItem -Path "${destination}\${leaf}" -Recurse | Move-Item -Destination "${destination}"
        Remove-Item -Path "${destination}\${leaf}"
    }
}

Function Disable-TomcatAJPConnector {
  Write-Host ""
  Write-Host "######################################################################################"
  Write-Host " Disabling Tomcat AJP connector within `"${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml`""
  Write-Host "######################################################################################"

  Set-Content `
    -Path "${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml" `
    -Value "$((Get-Content `"${DIR_BASE}\${DIR_TOMCAT}\conf\server.xml`" -Raw) -replace -join `
      (
        "(?ms)",
        "(\s+<!-- Define an AJP 1\.3 Connector on port [0-9]+ -->)(\s*)", # $1 $2
        "(<Connector port=``"[0-9]+``" protocol=``"AJP/1\.3``" redirectPort=``"[0-9]+``"\s*/>)(\s*)" # $3 $4
      ), -join `
      (
        "`$1`$2",
        "<!--`$2",
        "`$3`$2",
        "-->",
        "`$2`$2"
      )
    )"
}

Function Enable-CatalinaOutput {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Enable Output to CATALINA_OUT:"
    Write-Host "######################################################################################"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat" -ReadCount 0) -join "`r`n"
    $text -replace 'set _EXECJAVA=start "%TITLE%" "%_RUNJAVA%"', 'set _EXECJAVA=cmd /c "%_RUNJAVA%' | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat" -ReadCount 0) -join "`r`n"
    $text -replace '%ACTION%', '%ACTION%" >> %CATALINA_OUT% 2>&1' | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\catalina.bat"
}

Function Install-ActiveMQ {
    Unzip "${FILE_ACTIVEMQ}" "${DIR_BASE}\${DIR_ACTIVEMQ}"

    ((Get-Content -path "${DIR_BASE}\${DIR_ACTIVEMQ}\bin\activemq.bat" -Raw) -replace
        "if `"%JAVA_HOME%`" == `"`" goto noJavaHome",
        "if `"%JAVA_HOME%`" == `"`" set JAVA_HOME=${DIR_BASE}\jre") |
        Set-Content -Path "${DIR_BASE}\${DIR_ACTIVEMQ}\bin\activemq.bat"
}

Function Install-JDK {
    Unzip "${FILE_JDK}" "${DIR_BASE}\${DIR_JDK}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating symbolic link at ${DIR_BASE}\jre for ${DIR_JDK}\jre"
    Write-Host "######################################################################################"
    $null = (New-Item -ItemType SymbolicLink -Path "${DIR_BASE}\jre" -Target "${DIR_BASE}\${DIR_JDK}")
}

Function Install-PostgreSQL {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Installing PostgreSQL to ${DIR_BASE}\${DIR_POSTGRES}:"
    Write-Host "######################################################################################"
    Start-Process ${FILE_POSTGRESQL} -Wait -NoNewWindow -ArgumentList `
        "--unattendedmodeui none", `
        "--mode unattended", `
        "--superaccount ${NAME_POSTGRES}", `
        "--superpassword ${PARAM_POSTGRES_SECRET}", `
        "--serverport ${PARAM_POSTGRES_PORT}", `
        "--prefix ${DIR_BASE}\${DIR_POSTGRES}", `
        "--datadir ${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Updating PostgreSQL to allow remote access"
    Write-Host "######################################################################################"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}\pg_hba.conf" -ReadCount 0) -join "`n"
    $text -replace '/32', '/0 ' | Set-Content -Path "${DIR_BASE}\${DIR_POSTGRES}\${DIR_POSTGRES_DATA}\pg_hba.conf"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Restarting PostgreSQL"
    Write-Host "######################################################################################"
    Restart-Service -Name postgresql-x64-${VSN_POSTGRESQL_SERVICE}
}

Function Install-Tomcat {
    Unzip "$FILE_TOMCAT" "${DIR_BASE}\${DIR_TOMCAT}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Moving the default Tomcat wars to ${DIR_BASE}\${DIR_BACKUP}:"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_BASE}" -Name "${DIR_BACKUP}" -ItemType "directory")
    Get-ChildItem -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Recurse | Move-Item -Destination "${DIR_BASE}\${DIR_BACKUP}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating new default Tomcat index file at ${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\index.html:"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_BASE}\${DIR_TOMCAT}\webapps" -Name "ROOT" -ItemType "directory")
    Copy-Item -Path "${DIR_PRISTINE}\index.html" -Destination "${DIR_BASE}\${DIR_TOMCAT}\webapps\ROOT\index.html"

    Disable-TomcatAJPConnector
}

Function Join-ActiveMQTomcat {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Copying the ActiveMQ and log4j-api jar to ${DIR_BASE}\${DIR_TOMCAT}\lib:"
    Write-Host "######################################################################################"
    Copy-Item "${DIR_BASE}\${DIR_ACTIVEMQ}\${DIR_ACTIVEMQ}-all*.jar" -Destination "${DIR_BASE}\${DIR_TOMCAT}\lib"
    Copy-Item "${DIR_BASE}\${DIR_ACTIVEMQ}\lib\optional\log4j-api*.jar" -Destination "${DIR_BASE}\${DIR_TOMCAT}\lib"
}

Function Setup-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Prepare ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_LAUNCH}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_LAUNCH}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Prepare ${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_QUIT}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_BASE", "${DIR_BASE}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"
    $text = (Get-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}" -ReadCount 0) -join "`r`n"
    $text -replace "REPLACE_INSTALL_JAVA", "${DIR_BASE}\${DIR_JDK}" | Set-Content -Path "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_QUIT}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Create the Tomcat service ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Copy-Item -Path "${DIR_PRISTINE}\${NAME_TOMCAT_SERVICE}" -Destination "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE}"
    New-Service -Name "${SERV_TOMCAT_SERVICE}" -BinaryPathName "${DIR_BASE}\${DIR_TOMCAT}\bin\${NAME_TOMCAT_SERVICE} ${DIR_BASE}" -StartupType Manual
}

Function Restart-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Restart the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Restart-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Start-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Start the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Start-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Stop-TomcatService {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Stop the Tomcat service, ${SERV_TOMCAT_SERVICE}:"
    Write-Host "######################################################################################"
    Stop-Service -Name "${SERV_TOMCAT_SERVICE}"
}

Function Start-Recording {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Stopping existing transcripts"
    Write-Host "######################################################################################"
    $ErrorActionPreference="SilentlyContinue"
    Stop-Transcript | out-null

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Creating log directory at: ${DIR_LOG}"
    Write-Host "######################################################################################"
    $null = (New-Item -Path "${DIR_LOG_BASE}" -Name "${DIR_LOG_PARENT}" -ItemType "directory")

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Removing previous log if exists: ${FILE_LOG}"
    Write-Host "######################################################################################"
    Remove-Item -Path "${FILE_LOG}"

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Starting transcript: ${FILE_LOG}"
    Write-Host "######################################################################################"
    $ErrorActionPreference = "Continue"
    Start-Transcript -path "${FILE_LOG}" -append
}

Function Stop-Recording {
    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Complete the transcript"
    Write-Host "######################################################################################"
    Stop-Transcript

    Write-Host ""
    Write-Host "######################################################################################"
    Write-Host " Completed"
    Write-Host " Review the log file: ${FILE_LOG}"
    Write-Host "######################################################################################"
}

Start-Recording
If ($PARAM_INSTALL_JAVA) { Install-JDK }
If ($PARAM_INSTALL_ACTIVE_MQ) { Install-ActiveMQ }
If ($PARAM_INSTALL_WEB) {
    Install-Tomcat
    Enable-CatalinaOutput
    If ($PARAM_INSTALL_ACTIVE_MQ) { Join-ActiveMQTomcat }
    Setup-TomcatService
    If ($SERV_TOMCAT_STATE.equals("start")) { Start-TomcatService }
    ElseIf ($SERV_TOMCAT_STATE.equals("restart")) { Restart-TomcatService }
    ElseIf ($SERV_TOMCAT_STATE.equals("stop")) { Stop-TomcatService }
}
if ($PARAM_INSTALL_POSTGRESQL) { Install-PostgreSQL }
Stop-Recording


# SIG # Begin signature block
# MIIsvAYJKoZIhvcNAQcCoIIsrTCCLKkCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCfIvs8FiHRraXi
# d2HZtqXPabXVPXydbwRTPMTfvlRnBKCCJj8wggWNMIIEdaADAgECAhAOmxiO+dAt
# 5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAiBgNV
# BAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAwMDBa
# Fw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2Vy
# dCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lD
# ZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoC
# ggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3E
# MB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKy
# unWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsF
# xl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU1
# 5zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJB
# MtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObUR
# WBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6
# nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxB
# YKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5S
# UUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+x
# q4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6MIIB
# NjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwP
# TzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8EBAMC
# AYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdp
# Y2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNv
# bS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDigNoY0
# aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENB
# LmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCgv0Nc
# Vec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQTSnov
# Lbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh65Zy
# oUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSwuKFW
# juyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAOQGPF
# mCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjDTZ9z
# twGpn1eqXijiuZQwggWQMIIDeKADAgECAhAFmxtXno4hMuI5B72nd3VcMA0GCSqG
# SIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMx
# GTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRy
# dXN0ZWQgUm9vdCBHNDAeFw0xMzA4MDExMjAwMDBaFw0zODAxMTUxMjAwMDBaMGIx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBH
# NDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAL/mkHNo3rvkXUo8MCIw
# aTPswqclLskhPfKK2FnC4SmnPVirdprNrnsbhA3EMB/zG6Q4FutWxpdtHauyefLK
# EdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iTcMKyunWZanMylNEQRBAu34LzB4Tm
# dDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGbNOsFxl7sWxq868nPzaw0QF+xembu
# d8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclPXuU15zHL2pNe3I6PgNq2kZhAkHnD
# eMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCrVYJBMtfbBHMqbpEBfCFM1LyuGwN1
# XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFPObURWBf3JFxGj2T3wWmIdph2PVld
# QnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTvkpI6nj3cAORFJYm2mkQZK37AlLTS
# YW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWMcCxBYKqxYxhElRp2Yn72gLD76GSm
# M9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls5Q5SUUd0viastkF13nqsX40/ybzT
# QRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBRa2+xq4aLT8LWRV+dIPyhHsXAj6Kx
# fgommfXkaS+YHS312amyHeUbAgMBAAGjQjBAMA8GA1UdEwEB/wQFMAMBAf8wDgYD
# VR0PAQH/BAQDAgGGMB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qYrhwPTzANBgkq
# hkiG9w0BAQwFAAOCAgEAu2HZfalsvhfEkRvDoaIAjeNkaA9Wz3eucPn9mkqZucl4
# XAwMX+TmFClWCzZJXURj4K2clhhmGyMNPXnpbWvWVPjSPMFDQK4dUPVS/JA7u5iZ
# aWvHwaeoaKQn3J35J64whbn2Z006Po9ZOSJTROvIXQPK7VB6fWIhCoDIc2bRoAVg
# X+iltKevqPdtNZx8WorWojiZ83iL9E3SIAveBO6Mm0eBcg3AFDLvMFkuruBx8lbk
# apdvklBtlo1oepqyNhR6BvIkuQkRUNcIsbiJeoQjYUIp5aPNoiBB19GcZNnqJqGL
# FNdMGbJQQXE9P01wI4YMStyB0swylIQNCAmXHE/A7msgdDDS4Dk0EIUhFQEI6FUy
# 3nFJ2SgXUE3mvk3RdazQyvtBuEOlqtPDBURPLDab4vriRbgjU2wGb2dVf0a1TD9u
# KFp5JtKkqGKX0h7i7UqLvBv9R0oN32dmfrJbQdA75PQ79ARj6e/CVABRoIoqyc54
# zNXqhwQYs86vSYiv85KZtrPmYQ/ShQDnUBrkG5WdGaG5nLGbsQAe79APT0JsyQq8
# 7kP6OnGlyE0mpTX9iV28hWIdMtKgK1TtmlfB2/oQzxm3i0objwG2J5VT6LaJbVu8
# aNQj6ItRolb58KaAoNYes7wPD1N1KarqE3fk3oyBIa0HEEcRrYc9B9F1vM/zZn4w
# ggauMIIElqADAgECAhAHNje3JFR82Ees/ShmKl5bMA0GCSqGSIb3DQEBCwUAMGIx
# CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3
# dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBH
# NDAeFw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5NTlaMGMxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1
# c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0EwggIiMA0GCSqG
# SIb3DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPBPXJJUVXHJQPE8pE3qZdRodbS
# g9GeTKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/nR+eDzMfUBMLJnOWbfhXqAJ9
# /UO0hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLcZ47qUT3w1lbU5ygt69OxtXXn
# HwZljZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mfXazL6IRktFLydkf3YYMZ3V+0
# VAshaG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3NNg1c1eYbqMFkdECnwHLFuk4f
# sbVYTXn+149zk6wsOeKlSNbwsDETqVcplicu9Yemj052FVUmcJgmf6AaRyBD40Nj
# gHt1biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g3uM+onP65x9abJTyUpURK1h0
# QCirc0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD4L/wojzKQtwYSH8UNM/STKvv
# mz3+DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDSLFc1eSuo80VgvCONWPfcYd6T
# /jnA+bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwMO1uKIqjBJgj5FBASA31fI7tk
# 42PgpuE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU7s7pXcheMBK9Rp6103a50g5r
# mQzSM7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/BAgwBgEB/wIBADAdBgNVHQ4E
# FgQUuhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0jBBgwFoAU7NfjgtJxXWRM3y5n
# P+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMHcG
# CCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQu
# Y29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGln
# aUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8EPDA6MDigNqA0hjJodHRwOi8v
# Y3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNybDAgBgNV
# HSAEGTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwDQYJKoZIhvcNAQELBQADggIB
# AH1ZjsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPOvxj7x1Bd4ksp+3CKDaopafxp
# wc8dB+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQTGIdDAiCqBa9qVbPFXONASIl
# zpVpP0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWaeLJ7giqzl/Yy8ZCaHbJK9nXzQ
# cAp876i8dU+6WvepELJd6f8oVInw1YpxdmXazPByoyP6wCeCRK6ZJxurJB4mwbfe
# Kuv2nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfBwWpx2cYTgAnEtp/Nh4cku0+j
# Sbl3ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8lY5knLD0/a6fxZsNBzU+2QJsh
# IUDQtxMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/O3itTK37xJV77QpfMzmHQXh6
# OOmc4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbbbxV7HhmLNriT1ObyF5lZynDw
# N7+YAN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3OtUVmDG0YgkPCr2B2RP+v6TR
# 81fZvAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBldkKmKYcJRyvmfxqkhQ/8mJb2
# VVQrH4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt1nz8MIIGsDCCBJigAwIBAgIQ
# CK1AsmDSnEyfXs2pvZOu2TANBgkqhkiG9w0BAQwFADBiMQswCQYDVQQGEwJVUzEV
# MBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29t
# MSEwHwYDVQQDExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwHhcNMjEwNDI5MDAw
# MDAwWhcNMzYwNDI4MjM1OTU5WjBpMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGln
# aUNlcnQsIEluYy4xQTA/BgNVBAMTOERpZ2lDZXJ0IFRydXN0ZWQgRzQgQ29kZSBT
# aWduaW5nIFJTQTQwOTYgU0hBMzg0IDIwMjEgQ0ExMIICIjANBgkqhkiG9w0BAQEF
# AAOCAg8AMIICCgKCAgEA1bQvQtAorXi3XdU5WRuxiEL1M4zrPYGXcMW7xIUmMJ+k
# jmjYXPXrNCQH4UtP03hD9BfXHtr50tVnGlJPDqFX/IiZwZHMgQM+TXAkZLON4gh9
# NH1MgFcSa0OamfLFOx/y78tHWhOmTLMBICXzENOLsvsI8IrgnQnAZaf6mIBJNYc9
# URnokCF4RS6hnyzhGMIazMXuk0lwQjKP+8bqHPNlaJGiTUyCEUhSaN4QvRRXXegY
# E2XFf7JPhSxIpFaENdb5LpyqABXRN/4aBpTCfMjqGzLmysL0p6MDDnSlrzm2q2AS
# 4+jWufcx4dyt5Big2MEjR0ezoQ9uo6ttmAaDG7dqZy3SvUQakhCBj7A7CdfHmzJa
# wv9qYFSLScGT7eG0XOBv6yb5jNWy+TgQ5urOkfW+0/tvk2E0XLyTRSiDNipmKF+w
# c86LJiUGsoPUXPYVGUztYuBeM/Lo6OwKp7ADK5GyNnm+960IHnWmZcy740hQ83eR
# Gv7bUKJGyGFYmPV8AhY8gyitOYbs1LcNU9D4R+Z1MI3sMJN2FKZbS110YU0/EpF2
# 3r9Yy3IQKUHw1cVtJnZoEUETWJrcJisB9IlNWdt4z4FKPkBHX8mBUHOFECMhWWCK
# ZFTBzCEa6DgZfGYczXg4RTCZT/9jT0y7qg0IU0F8WD1Hs/q27IwyCQLMbDwMVhEC
# AwEAAaOCAVkwggFVMBIGA1UdEwEB/wQIMAYBAf8CAQAwHQYDVR0OBBYEFGg34Ou2
# O/hfEYb7/mF7CIhl9E5CMB8GA1UdIwQYMBaAFOzX44LScV1kTN8uZz/nupiuHA9P
# MA4GA1UdDwEB/wQEAwIBhjATBgNVHSUEDDAKBggrBgEFBQcDAzB3BggrBgEFBQcB
# AQRrMGkwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBBBggr
# BgEFBQcwAoY1aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1
# c3RlZFJvb3RHNC5jcnQwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybDMuZGln
# aWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZFJvb3RHNC5jcmwwHAYDVR0gBBUwEzAH
# BgVngQwBAzAIBgZngQwBBAEwDQYJKoZIhvcNAQEMBQADggIBADojRD2NCHbuj7w6
# mdNW4AIapfhINPMstuZ0ZveUcrEAyq9sMCcTEp6QRJ9L/Z6jfCbVN7w6XUhtldU/
# SfQnuxaBRVD9nL22heB2fjdxyyL3WqqQz/WTauPrINHVUHmImoqKwba9oUgYftzY
# gBoRGRjNYZmBVvbJ43bnxOQbX0P4PpT/djk9ntSZz0rdKOtfJqGVWEjVGv7XJz/9
# kNF2ht0csGBc8w2o7uCJob054ThO2m67Np375SFTWsPK6Wrxoj7bQ7gzyE84FJKZ
# 9d3OVG3ZXQIUH0AzfAPilbLCIXVzUstG2MQ0HKKlS43Nb3Y3LIU/Gs4m6Ri+kAew
# Q3+ViCCCcPDMyu/9KTVcH4k4Vfc3iosJocsL6TEa/y4ZXDlx4b6cpwoG1iZnt5Lm
# Tl/eeqxJzy6kdJKt2zyknIYf48FWGysj/4+16oh7cGvmoLr9Oj9FpsToFpFSi0HA
# SIRLlk2rREDjjfAVKM7t8RhWByovEMQMCGQ8M4+uKIw8y4+ICw2/O/TOHnuO77Xr
# y7fwdxPm5yg/rBKupS8ibEH5glwVZsxsDsrFhsP2JjMMB0ug0wcCampAMEhLNKhR
# ILutG4UI4lkNbcoFUCvqShyepf2gpx8GdOfy1lKQ/a+FSCH5Vzu0nAPthkX0tGFu
# v2jiJmCG6sivqf6UHedjGzqGVnhOMIIGwjCCBKqgAwIBAgIQBUSv85SdCDmmv9s/
# X+VhFjANBgkqhkiG9w0BAQsFADBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGln
# aUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5
# NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMB4XDTIzMDcxNDAwMDAwMFoXDTM0MTAx
# MzIzNTk1OVowSDELMAkGA1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMu
# MSAwHgYDVQQDExdEaWdpQ2VydCBUaW1lc3RhbXAgMjAyMzCCAiIwDQYJKoZIhvcN
# AQEBBQADggIPADCCAgoCggIBAKNTRYcdg45brD5UsyPgz5/X5dLnXaEOCdwvSKOX
# ejsqnGfcYhVYwamTEafNqrJq3RApih5iY2nTWJw1cb86l+uUUI8cIOrHmjsvlmbj
# aedp/lvD1isgHMGXlLSlUIHyz8sHpjBoyoNC2vx/CSSUpIIa2mq62DvKXd4ZGIX7
# ReoNYWyd/nFexAaaPPDFLnkPG2ZS48jWPl/aQ9OE9dDH9kgtXkV1lnX+3RChG4PB
# uOZSlbVH13gpOWvgeFmX40QrStWVzu8IF+qCZE3/I+PKhu60pCFkcOvV5aDaY7Mu
# 6QXuqvYk9R28mxyyt1/f8O52fTGZZUdVnUokL6wrl76f5P17cz4y7lI0+9S769Sg
# LDSb495uZBkHNwGRDxy1Uc2qTGaDiGhiu7xBG3gZbeTZD+BYQfvYsSzhUa+0rRUG
# FOpiCBPTaR58ZE2dD9/O0V6MqqtQFcmzyrzXxDtoRKOlO0L9c33u3Qr/eTQQfqZc
# ClhMAD6FaXXHg2TWdc2PEnZWpST618RrIbroHzSYLzrqawGw9/sqhux7UjipmAmh
# cbJsca8+uG+W1eEQE/5hRwqM/vC2x9XH3mwk8L9CgsqgcT2ckpMEtGlwJw1Pt7U2
# 0clfCKRwo+wK8REuZODLIivK8SgTIUlRfgZm0zu++uuRONhRB8qUt+JQofM604qD
# y0B7AgMBAAGjggGLMIIBhzAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIwADAW
# BgNVHSUBAf8EDDAKBggrBgEFBQcDCDAgBgNVHSAEGTAXMAgGBmeBDAEEAjALBglg
# hkgBhv1sBwEwHwYDVR0jBBgwFoAUuhbZbU2FL3MpdpovdYxqII+eyG8wHQYDVR0O
# BBYEFKW27xPn783QZKHVVqllMaPe1eNJMFoGA1UdHwRTMFEwT6BNoEuGSWh0dHA6
# Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZTSEEy
# NTZUaW1lU3RhbXBpbmdDQS5jcmwwgZAGCCsGAQUFBwEBBIGDMIGAMCQGCCsGAQUF
# BzABhhhodHRwOi8vb2NzcC5kaWdpY2VydC5jb20wWAYIKwYBBQUHMAKGTGh0dHA6
# Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNFJTQTQwOTZT
# SEEyNTZUaW1lU3RhbXBpbmdDQS5jcnQwDQYJKoZIhvcNAQELBQADggIBAIEa1t6g
# qbWYF7xwjU+KPGic2CX/yyzkzepdIpLsjCICqbjPgKjZ5+PF7SaCinEvGN1Ott5s
# 1+FgnCvt7T1IjrhrunxdvcJhN2hJd6PrkKoS1yeF844ektrCQDifXcigLiV4JZ0q
# BXqEKZi2V3mP2yZWK7Dzp703DNiYdk9WuVLCtp04qYHnbUFcjGnRuSvExnvPnPp4
# 4pMadqJpddNQ5EQSviANnqlE0PjlSXcIWiHFtM+YlRpUurm8wWkZus8W8oM3NG6w
# QSbd3lqXTzON1I13fXVFoaVYJmoDRd7ZULVQjK9WvUzF4UbFKNOt50MAcN7MmJ4Z
# iQPq1JE3701S88lgIcRWR+3aEUuMMsOI5ljitts++V+wQtaP4xeR0arAVeOGv6wn
# LEHQmjNKqDbUuXKWfpd5OEhfysLcPTLfddY2Z1qJ+Panx+VPNTwAvb6cKmx5Adza
# ROY63jg7B145WPR8czFVoIARyxQMfq68/qTreWWqaNYiyjvrmoI1VygWy2nyMpqy
# 0tg6uLFGhmu6F/3Ed2wVbK6rr3M66ElGt9V/zLY4wNjsHPW2obhDLN9OTH0eaHDA
# dwrUAuBcYLso/zjlUlrWrBciI0707NMX+1Br/wd3H3GXREHJuEbTbDJ8WC9nR2Xl
# G3O2mflrLAZG70Ee8PBf4NvZrZCARK+AEEGKMIIG6jCCBNKgAwIBAgIQBKbQDKbn
# vxPWBgHfq1yGCTANBgkqhkiG9w0BAQsFADBpMQswCQYDVQQGEwJVUzEXMBUGA1UE
# ChMORGlnaUNlcnQsIEluYy4xQTA/BgNVBAMTOERpZ2lDZXJ0IFRydXN0ZWQgRzQg
# Q29kZSBTaWduaW5nIFJTQTQwOTYgU0hBMzg0IDIwMjEgQ0ExMB4XDTIxMDYwODAw
# MDAwMFoXDTI0MDgzMDIzNTk1OVowbzELMAkGA1UEBhMCR0IxEDAOBgNVBAcTB05F
# V0JVUlkxJjAkBgNVBAoTHU1pY3JvIEZvY3VzIEludGVybmF0aW9uYWwgcGxjMSYw
# JAYDVQQDEx1NaWNybyBGb2N1cyBJbnRlcm5hdGlvbmFsIHBsYzCCAaIwDQYJKoZI
# hvcNAQEBBQADggGPADCCAYoCggGBAOoiQYhGCwyc+fGmIadVulPc2GZ9lCYQxC9K
# 1vSQ3pqs+3huyMrjKp2tXRU4HmwpY2w3tdvYr7Zpbj3wbKPR67IBp0IFvz0KNAYt
# 7AebJHbdPOqZvS01XV5ofNq59ABKvQOAcuAdmUadYLdVRYcfdMDX4D2i0WSuheIS
# NkZW3WrpBDYeRryJ2HwhoSQXbhHGtfihGEQHtz+KLpjSxJ/ntj60epd1i6a4Q2Ro
# Djgtrv0Ee4xq/ATZ9a6ukTuWB+V/WedvInFpD5etSBKqLhC93pnCPoO+jmlQkC6q
# qHApjAfSfsECnEUqVmr/DpR1LU1MyiGbHKog3LED7h0pCKUpQdeJc/R34ZRKtEW1
# e01TdMqCk7YsUZlKWz+Xbl6ZzYidPaYdPtKQaSXAPzuIzbys9LoKd5U2VBxCylLL
# vZgJEQlYhXUlu/vEBek6XzBuBUH4DZObmVIbkVS7v2sT6fXik6ibhU4PPEDdq0yG
# bjSHfze1LpXc8fIriotRukuSRIMv8QIDAQABo4ICBjCCAgIwHwYDVR0jBBgwFoAU
# aDfg67Y7+F8Rhvv+YXsIiGX0TkIwHQYDVR0OBBYEFIboW+tVaDsOcwhEgehx7b3u
# JpoxMA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzCBtQYDVR0f
# BIGtMIGqMFOgUaBPhk1odHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRU
# cnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQyMDIxQ0ExLmNybDBToFGg
# T4ZNaHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0Q29k
# ZVNpZ25pbmdSU0E0MDk2U0hBMzg0MjAyMUNBMS5jcmwwPgYDVR0gBDcwNTAzBgZn
# gQwBBAEwKTAnBggrBgEFBQcCARYbaHR0cDovL3d3dy5kaWdpY2VydC5jb20vQ1BT
# MIGUBggrBgEFBQcBAQSBhzCBhDAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGln
# aWNlcnQuY29tMFwGCCsGAQUFBzAChlBodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5j
# b20vRGlnaUNlcnRUcnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQyMDIx
# Q0ExLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBCwUAA4ICAQBdyv7VGFvF
# 12qOAQ3bV8ae6galTCD2mxwlzEfc+6cmh/5h/h9N8f6THKy97LsrDN+wWA4AaVfL
# 4o0Pk/lZI4FXYFC5ljqwoFIA2CqcqVi2fpnWSlDcNSu+WLIrcD9StY75xkEPvgnV
# sMbLyIhQvlMwMpQh/MnMREQ40I/3nXZDVksjoRa3frEQYQLr8wEmHoS86l/tJ4R0
# P/cmK63nGMb8uc1LgREopxlCjf/NaNK8z2BjD8iJ7E0B1pGyr5viRM0/+I+OjAru
# bsmWUcYPkibX7pQ1c1shDfpu4fo3eqSz2nIMPEnKOeJrxSvi5tXfY8mIfO4uo43K
# Dg+9SZgWTLe4T2P1/jERijc3SpiF9QOYpDjktQPBla4CcF6b5HAi/ffs35T7wCNq
# mlA9hDrb+6djoCHkKaCQbbWklPujN/SxR2jpfkrEX0QuwZ76dZ9RYh9aNFeSLw4r
# qH6Lf2Bd2JjYTpw1zKN+FB6x44dZqWmVzFSrxxnzxxS1ptuwwyzL6dRpDO003y0O
# bQV2sCoUb3jeE37ifqOOfqeSrsIqueFCwlHtCnLjKM6KTR8iykYvoXop8UeADEJY
# Yyi8tqt3SbCKBF4fTEjLq82CUbpjjlEqVoN8pXyoFZyt9QoXE5bsygERRyPn07Mg
# fMhL3Zj6/nYG8FzyJe9YyjHD7smoGhwl9DGCBdMwggXPAgEBMH0waTELMAkGA1UE
# BhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMUEwPwYDVQQDEzhEaWdpQ2Vy
# dCBUcnVzdGVkIEc0IENvZGUgU2lnbmluZyBSU0E0MDk2IFNIQTM4NCAyMDIxIENB
# MQIQBKbQDKbnvxPWBgHfq1yGCTANBglghkgBZQMEAgEFAKCBhDAYBgorBgEEAYI3
# AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisG
# AQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCBMaLJmR9LY
# LDLnwvkUx2seSncm7vn2q5HFbGhCeKtrhDANBgkqhkiG9w0BAQEFAASCAYBSBz7d
# 2KvGXN8mtrZJ6FDtk4HrE8u6G5uAfDdlfOoCfmU7vhw2aTV1pw6P1I2RlW4WDeFK
# WzWjWrLasJ5ZPjhu8ZdsKvDZ0DKYwxPemWvZSK0aXQoUjilWObLtC4E99mc86hP2
# xABnNWknevtqgI7DAm3b44Uzhor4JN50iTNJN+kcMgIALVoN8ehmhHxyeFXAvY/W
# dOPQTDdQVI2Ds1YIjNC6N9EFdl2dLDU1f1AtnNkDNWUK4L9eYAfA/4vM/dunyDJ8
# harhJyEh9pV3N3R5n9rU4o+X0D5vSsoCWKVxREH31spxAXkrBqCXwFMmg25wGCsP
# LnAZYOf3P5hUMPMOhsOjxrAHo19NtcyzoBOoDpDYsGaeLlx7qZxQz18j63Zemz/g
# pZSc3MxaYs7XkOPStw4p7tu2Gk89AUGi2lkWx17PHfvcxDtVmByi9oGHvyK3V1vH
# 8RkGkykhTGc0oa8UOgm7DQX7jZMzaTUzdqy+pFxxmfr4QHVKFK7s4JDsIWShggMg
# MIIDHAYJKoZIhvcNAQkGMYIDDTCCAwkCAQEwdzBjMQswCQYDVQQGEwJVUzEXMBUG
# A1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQg
# RzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBAhAFRK/zlJ0IOaa/2z9f
# 5WEWMA0GCWCGSAFlAwQCAQUAoGkwGAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAc
# BgkqhkiG9w0BCQUxDxcNMjQwMTA1MTM0NTA1WjAvBgkqhkiG9w0BCQQxIgQg0wrM
# MltWqGSZMJLgmKzL4KMoeVLWbAeu6DUV/S4M0pcwDQYJKoZIhvcNAQEBBQAEggIA
# DptDmjVf/J0ex6Nvrkfe6gT3y2wZMYLJXNh3KkJzmP+ITfpuLPGrHsbzD1ffm+rY
# NTnuu3naJ4O7d5VVb3ZKu/zzTMq+g+tEvNebkh9+x8hi0Vl/SleaDLJMgimkfeSR
# NkLIXUnHCZo5Em5blRvj89iqcqP4gEI6QfeISnvHWIDjzu3hbSzJB3QJEVTXRuV8
# nZr54iudJzGQvP+xd5JQykzP3CyWdrcTsC8Fu6tFMWwB/OfKfQnYd4dsg+7bgoU+
# KHF2RHLDZEUNmwHZ3WcSwA1esF8zBoKwBY/qbP8Hqzc9dNyDAp4m0ywQ7NYuGm9U
# se08SI8d/MWE2pIZTyJ+e+dWpvkv249FV2ALLr0CvVNgYj6LkUHc8WvdhHtEGGeK
# ZzIGsI9EDudmpRqqQOZDGYw2UpVDJMKEWlct6bMROtwi1Iewc74Fxa8BMYtu6SIb
# Zp8H/jWYQLbj3FHJMcK/zWFPeJMjL2XimtUS/jJfoHPa3hdNUq/feZbGA5wd5aCg
# cryEvrBhzfFqT0/L/8EZo6M7LTqYred/z/Fpzfd8+ky0vjzNtCLzyxOyhWLKE4Mm
# 7AoF05HLTIxL7pikhHjVrvJLhUygxK1p7wCMG83dvHu9Tlb5uJ2hU2B+WwFwNK2z
# 8ohQ97PglQIICTuTtopKuNs1O3sXFXe8oPFfm4ka6nE=
# SIG # End signature block
